/*
 * Titolo progetto:         DJ Controller per webradio
 * Autore ultima modifica:  Sarga Luca (Fablab Bergamo - RadioPaneSalame)
 * Contatti:                luca.sarga@fablabbergamo.it   www.fablabbergamo.it  www.radiopanesalame.it
 * 
 * This work is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.
 * See http://creativecommons.org/licenses/by-nc-sa/3.0/ for license details.
 * 
 * Cosa serve: -hardware    1 x Arduino Uno R3
 *                          3 x Potenziometro 10 kOhm (volume deck 1, volume deck 2, volume fader)
 *                          2 x Pulsanti (play/pause deck 1, play/pause deck 2)
 *                          2 x Resistenza 4.7 kOhm
 *                          2 x Led rosso 20mA
 *                          2 x Resistenza da 220 Ohm
 *                          cavi vari
 *             -software    Arduino IDE (www.arduino.cc), 
 *                          Hariless Midi Serial (http://projectgus.github.io/hairless-midiserial/), 
 *                          LoopBe1 (http://www.nerds.de/en/loopbe1.html)
 *             
 * Progetti ispiratrici:    Arduino MIDI Controller by dylanstout (http://www.instructables.com/id/Arduino-MIDI-Controller/)         
 *                          Arcade Button MIDI Controller by fraganator  (http://www.instructables.com/id/Arcade-Button-MIDI-Controller/)
 *                          
 * Convenzione di nominazione >> Deck A è il player di sinistra; Deck B è il player di destra
 */

/*  Inseriamo i valori validi per il funzionamento dei pulsanti */
#define LEDA 13                // LED collegato al pin digitale 13  
#define BUTTONA 7              // pin di input dove è collegato il pulsante  
int valA = 0;                  // si userà val per conservare lo stato del pin di input  
int vecchio_valA = 0;          // si userà vecchio_val per conservare lo stato del pin di input al passo precedente  
int statoA = 0;                // ricorda lo stato in cui si trova il led, stato = 0 led spento, stato = 1 led acceso  
int volatileA = 0;             // variabile volatile per il deck A

#define LEDB 12                // LED collegato al pin digitale 13  
#define BUTTONB 6              // pin di input dove è collegato il pulsante  
int valB = 0;                  // si userà val per conservare lo stato del pin di input  
int vecchio_valB = 0;          // si userà vecchio_val per conservare lo stato del pin di input al passo precedente  
int statoB = 0;                // ricorda lo stato in cui si trova il led, stato = 0 led spento, stato = 1 led acceso  
int volatileB = 0;             // variabile volatile per il deck B

//  Inseriamo i valori di partenza pari a 0 per ogni potenziometro, sia del valore attuale che del valore precedente. 
//  Questo ci serve per verificare il cambiamento di stato del potenziometro.
int val = 0;        //valore primo potenziometro
int lastVal = 0;    //ultimo valore registrato dal primo potenziometro
int val2 = 0;       //valore secondo potenziometro
int lastVal2 = 0;   //ultimo valore registrato dal secondo potenziometro
int val3 = 0;       //valore terzo potenziometro
int lastVal3 = 0;   //ultimo valore registrato dal terzo potenziometro
  
void setup() {  
  Serial.begin(9600);          // Impostare il bitrate della seriale per utilizzare il software Hairless Midi Serial 
  pinMode(LEDA, OUTPUT);       // imposta il pin digitale come output  
  pinMode(BUTTONA, INPUT);     // imposta il pin digitale come input  
  pinMode(LEDB, OUTPUT);       // imposta il pin digitale come output  
  pinMode(BUTTONB, INPUT);     // imposta il pin digitale come input  
}  
  
void loop() {  
     /*
    * SETTORE POTENZIOMETRI
    * Qui inseriamo le impostazioni per i tre potenziometri, rispettivamente connessi a 
    * A0 - volume deck A
    * A1 - volume deck B
    * A3 - volume cross-fader
    */

// Potenziometro volume deck A

   val = analogRead(0)/8;     // Dividi l'input ricevuto da A0 per 8, in modo da avere valori compresi nel range da 0 a 127 per la corretta conversione MIDI
   if (val != lastVal)        // Se (if) il valore della variabile "val" non è uguale al valore precedente "lastVal" invia il seguente comando. Questo perchè abbiamo un cambiamento di stato del potenziometro. Se invece non c'è il cambiamento di stato allora non invieremo nulla sulla porta MIDI.
   {
   MIDImessage(176,1,val);}   // 176 = CC command (channel 1 control change), 1 = potenziometro 1, val = valore letto dal primo potenziometro
   lastVal = val;             // registriamo il valore appena inviato per le prossime iterazioni di cambiamento di stato


// Potenziometro volume deck B

   val2 = analogRead(1)/8;    // Dividi l'input ricevuto da A1 per 8, in modo da avere valori compresi nel range da 0 a 127 per la corretta conversione MIDI
   if (val2 != lastVal2) 
   {
   MIDImessage(176,2,val2);}  // 176 = CC command, 2 = potenziometro 2, val = valore letto dal secondo potenziometro
   lastVal2 = val2;


// Potenziometro cross-fader

   val3 = analogRead(2)/8;    // Dividi l'input ricevuto da A1 per 8, in modo da avere valori compresi nel range da 0 a 127 per la corretta conversione MIDI
   if (val3 != lastVal3) 
   {
   MIDImessage(176,3,val3);}  // 176 = CC command, 3 = potenziometro 3, val = valore letto dal secondo potenziometro
   lastVal3 = val3;

/* SETTORE PULSANTI DI CONTROLLO
* Qui si inderiranno le variabili dei pulsanti a cui affidiamo delle funzioni particolari del nostro deck, ovvero play/pause, stop, effetti, etc.
* In questa prima bozza inseriamo solamente i pulsanti play/pause relativi ai deck A e B, associati ad un led che ci indicherà lo stato del player.
 */
 
  valA = digitalRead(BUTTONA);  // legge il valore dell'input e lo conserva  
  
  // controlla se è accaduto qualcosa  
  if ((valA == HIGH) && (vecchio_valA == LOW)){  
    statoA = 1 - statoA;  
    delay(15);                  // attesa di 15 millisecondi  
  }   
  
  vecchio_valA = valA;          // ricordiamo il valore precedente di valA per il controllo successivo
  
    if (statoA == 1) {  
    digitalWrite(LEDA, HIGH);             // accende il LED del deck A - stato di play
          if ((volatileA != 1)){
            MIDImessage(176,4,127);       // 176 = CC command, 4 = pulsante del deck A, 127 = play
            volatileA = 1;}               // la mia variabile volatile passa a stato 1 
    } 
    else {  
    digitalWrite(LEDA, LOW);              //spegne il LED del deck A - stato di pause
          if ((volatileA != 0)){
          MIDImessage(176,4,0);           // 176 = CC command, 4 = pulsante del deck A, 0 = stop  
          volatileA = 0;}                 // la mia variabile volatile passa a stato 0
    }  

    valB = digitalRead(BUTTONB);          // legge il valore dell'input e lo conserva  
  
  // controlla se è accaduto qualcosa  
  if ((valB == HIGH) && (vecchio_valB == LOW)){  
    statoB = 1 - statoB;  
    delay(15);                            // attesa di 15 millisecondi  
  }   
  
  vecchio_valB = valB;                    // ricordiamo il valore precedente di valB per il controllo successivo
  
    if (statoB == 1) {  
    digitalWrite(LEDB, HIGH);             // accende il LED del deck B - stato di play  
      if ((volatileB != 1)){
            MIDImessage(176,5,127);       // 176 = CC command, 4 = pulsante del deck A, 127 = play
            volatileB = 1;}               // la mia variabile volatile passa a stato 1 
  }  
  else {  
    digitalWrite(LEDB, LOW);              //spegne il LED del deck B - stato di pause  
    if ((volatileB != 0)){
          MIDImessage(176,5,0);           // 176 = CC command, 4 = pulsante del deck A, 0 = stop  
          volatileB = 0;}                 // la mia variabile volatile passa a stato 0
  }  
delay(10);                    //si aggiunge un piccolo ritardo (10ms) per ridurre flutuazioni dovute allo spostamento lieve ed involontario del potenziometro 
}

/* CONVERSIONE DEL LINGUAGGIO
 * Tutto il codice sopra riportato termina con un MIDImessage, una stringa che contiene 3 valori che servono alla comunicazione MIDI dei programmi (nel nostro caso
 * il software open mixxx.
 * La stinga di seguito scrive i valori in seriale che verranno poi letti in modo corretto dalla porta virtuale MIDI generata con LoopBe1.
 */

void MIDImessage(byte command, byte data1, byte data2)  //conversione nel linguaggio MIDI
{
   Serial.write(command);
   Serial.write(data1);
   Serial.write(data2);
}

